#!/bin/sh
#-*-tcl-*-
# the next line restarts using tclsh \
exec tclsh "$0" ${1+"$@"}

# Driver installer script for WWPass token and reader for Apple macOsX and Linux
# Author: Mikhail Vysogorets m.vysogorets@wwpass.com
# Copyright WWPass Corp http://wwpass.com
# usage: just run in superuser mode, e.g.
#sudo ./wwpass_rdr_install.tcl
# it may be required to add execution caps to the file:
#chmod a+x wwpass_rdr_install.tcl

set new_devices {
    {0x21BB 0x0002 "WWPass Key"}
    {0x21BB 0x0003 "WWPass Reader"}
}

proc find_files {} {
    global in_filename out_file argv tcl_platform

    if { [llength $argv] > 0 } {
        set in_filename [lindex $argv 0]
        return 1;
    }
    if {  $tcl_platform(os) == "Darwin" } {
        set in_filename "/usr/libexec/SmartCardServices/drivers/ifd-ccid.bundle/Contents/Info.plist"
        return 1
    }
    if {  $tcl_platform(os) == "Linux" } {
        set in_filename "/etc/libccid_Info.plist";
        return 1
    }
    return 0
}

set ifdVendorID [list ]
set ifdProductID [list ]
set ifdFriendlyName [list ]

proc isnew { vid pid } {
    global new_devices

    for { set i 0 } { $i < [llength $new_devices] } {incr i } {
        set d [lindex $new_devices $i]
        if { $vid == [lindex $d 0] } {
            if { $pid == [lindex $d 1] } {
                return $i
            }
        }
    }
    return -1
}

proc analize_scan {} {
    global ifdVendorID ifdProductID ifdFriendlyName new_devices fnew

#    puts "[llength $ifdVendorID] [llength $ifdProductID] [llength $ifdFriendlyName]"

    if { [llength $ifdVendorID] != [llength $ifdProductID] } {
        error " ifdVendorID and ifdProductID lengths differs"

    }
    if { [llength $ifdProductID] != [llength $ifdFriendlyName]} {
        error "file corrupted: ifdProductID and ifdFriendlyName lengths differs"
    }
    if { [llength $ifdProductID] ==0 } {
        error "empty ifdProductID list"
    }

    set nd $new_devices

    set rows [llength $ifdProductID]
    for { set i 0 } { $i < $rows } { incr i } {
        set n [isnew [lindex $ifdVendorID $i] [lindex $ifdProductID $i]]
        if { $n >= 0} {
            set ifdFriendlyName [lreplace $ifdFriendlyName $i $i [lindex $new_devices $n 2]]
            set new_devices [lreplace $new_devices $n $n]
        }
    }
    return [ llength $new_devices ]
}

proc write_arrays {} {
    global ifdVendorID ifdProductID ifdFriendlyName new_devices fnew

    set rows [llength $ifdProductID]

    if 0 {
        for { set i 0 } { $i < $rows } { incr i } {
            set n [isnew [lindex $ifdVendorID $i] [lindex $ifdProductID $i]]
            if { $n >= 0} {
                set ifdFriendlyName [lreplace $ifdFriendlyName $i $i [lindex $new_devices $n 2]]
                set new_devices [lreplace $new_devices $n $n]
            }
        }
    }

    foreach d $new_devices {
        lappend ifdVendorID [lindex $d 0]
        lappend ifdProductID [lindex $d 1]
        lappend ifdFriendlyName [lindex $d 2]
    }

    set rows [llength $ifdProductID]

    for { set i 0 } { $i < $rows } { incr i } {
#        puts "[lindex $ifdVendorID $i] [lindex $ifdProductID $i] [lindex $ifdFriendlyName $i]"
    }

    puts $fnew  "\t<key>ifdVendorID</key>"
    puts $fnew  "\t<array>"
    for { set i 0 } { $i < $rows } { incr i } {
        puts $fnew "\t\t<string>[lindex  $ifdVendorID $i]</string>"
    }
    puts $fnew  "\t</array>"
    puts $fnew  ""

    puts $fnew  "\t<key>ifdProductID</key>"
    puts $fnew  "\t<array>"
    for { set i 0 } { $i < $rows } { incr i } {
        puts $fnew "\t\t<string>[lindex  $ifdProductID $i]</string>"
    }
    puts $fnew  "\t</array>"
    puts $fnew  ""

    puts $fnew  "\t<key>ifdFriendlyName</key>"
    puts $fnew  "\t<array>"
    for { set i 0 } { $i < $rows } { incr i } {
        puts $fnew "\t\t<string>[lindex  $ifdFriendlyName $i]</string>"
    }
    puts $fnew  "\t</array>"
}

proc scan { do_write } {
    global in_filename ifdVendorID ifdProductID ifdFriendlyName fnew argv0

    set state "transparent"
    set ifdVendorID [list ]
    set ifdProductID [list ]
    set ifdFriendlyName [list ]

    set in_file [open $in_filename]

    if { $do_write  == 0 } {
        set in_file [open $in_filename]
    } else {
        if [ catch {
            file rename -force $in_filename "${in_filename}.bak"
            set in_file [open "${in_filename}.bak"]
            set fnew [open $in_filename "w"]
        } msg ] {
            puts "permission denied, run as\nsudo $argv0"
            return
        }
    }

    while { 1 } {
        gets $in_file line

        if [eof $in_file ] {
            close $in_file
            break
        }
        if [regexp "<key>\\s*ifdVendorID\\s*</key>" $line] {
            if { $state != "ifdVendorID" } {
                set state "ifdVendorID"
            }
        } elseif [regexp "<key>\\s*ifdProductID\\s*</key>" $line] {
            if { $state != "ifdProductID" } {
                set state "ifdProductID"
            }
        } elseif [regexp "<key>\\s*ifdFriendlyName\\s*</key>" $line] {
            if { $state != "ifdFriendlyName" } {
                set state "ifdFriendlyName"
            }
        } elseif [regexp "^\\s*</array>\\s*$" $line] {
            if { [llength $ifdFriendlyName] } {
                set state "transparent"
                if { $do_write == 0 } {
                    close $in_file
                    return 1
                }
                write_arrays
            }
        }

        if { $state == "transparent" } {
            if { [regexp "^\\s*</array>\\s*$" $line] == 0 } {
                if { $do_write } {
                    puts $fnew $line
                }
            }
        } elseif [regexp "<string>\\s*(.*)\\s*</string>" $line id0 id] {
            if { $state == "ifdVendorID" } {
                lappend ifdVendorID $id
            }
            if { $state == "ifdProductID" } {
                lappend ifdProductID $id
            }
            if { $state == "ifdFriendlyName" } {
                lappend ifdFriendlyName $id
            }
        }
    }
}

proc main {} {
    global in_filename new_devices
    find_files
    puts "config file $in_filename"
    scan 0

    if [catch { analize_scan } result] {
        puts $result
        exit
    }
    set add_count [llength $new_devices]

    if { $add_count ==0 } {
        puts "$in_filename already updated"
        exit
    }
    puts "$add_count device(s) will be added"
    foreach x $new_devices {
        puts "\t$x"
    }
    scan 1
}

main

exit
